#include                <stdio.h>
#include                <unistd.h>
#include                <fcntl.h>
#include                <stdlib.h>
#include                <string.h>
#include                <sys/stat.h>
#include                <sys/mount.h>

#include                "tap.h"
#include                "libFireBird.h"
#include                "TMSCommander.h"

#define PROGRAM_NAME    "TMSMount"
#define VERSION         "V0.8a"

#define DEFAULTMOUNTPOINT   "/mnt/hd/DataFiles/external"
#define BUFFERSIZE      32768
#define INIFILE         "mount.txt"
#define WAITTIMEOUT     6000

TAP_ID                  (0x8E0A423C);
TAP_PROGRAM_NAME        (PROGRAM_NAME" "VERSION);
TAP_PROGRAM_VERSION     (VERSION);
TAP_AUTHOR_NAME         ("FireBird");
TAP_DESCRIPTION         ("TMSMount");
TAP_ETCINFO             (__DATE__);

typedef enum
{
  FST_Normal,
  FST_VFAT,
  FST_NTFS,
  FST_EXT2,
  FST_EXT3
} tFileSystemType;

typedef enum
{
  ST_Init,
  ST_WaitForDisk,
  ST_MountDisk,
  ST_ShowSuccessMessage,
  ST_ShowFailMessage,
  ST_Idle,
  ST_Exit,
} tState;

char                    DeviceName[20];     // /dev/sdb1
char                    OrigMountPoint[20]; // /mnt/sdb1
char                    UserMountPoint[255]; // /mnt/hd/DataFiles/external
tFileSystemType         FileSystemType = FST_Normal;
tState                  State = ST_Init;
char                    Buffer[255];
dword                   WaitTimeout;

long get_filesize(char *FileName)
{
  struct stat           statbuf;

  if(stat(FileName, &statbuf)) return 0;

  return (dword)statbuf.st_size;
}

bool GetCurrentMountInfo(void)
{
  char                 *Buffer;
  char                 *p, *q, *EOL;
  char                  c;
  char                  DevName[12];
  int                   fd;

  //Read the current mount points
  Buffer = TAP_MemAlloc(BUFFERSIZE);
  system("mount > /root/mnt.log");
  TAP_Delay(10);
  fd = open("/root/mnt.log", O_RDONLY);
  read(fd, Buffer, BUFFERSIZE);
  close(fd);
  Buffer[BUFFERSIZE - 1] = '\0';

  for(c = 'b'; c < 'g'; c++)
  {
    TAP_SPrint(DevName, "/dev/sd%c", c);
    p = strstr(Buffer, DevName);
    if(p)
    {
      //Terminate at the end of the line and search for type vfat
      EOL = strpbrk(p, "\x0d\x0a\x00");
      if(EOL) *EOL = '\0';

      if(strstr(p, "type vfat") != 0)    FileSystemType = FST_VFAT;
      if(strstr(p, "type fuseblk") != 0) FileSystemType = FST_NTFS;
      if(strstr(p, "type ext2") != 0) FileSystemType = FST_EXT2;
      if(strstr(p, "type ext3") != 0) FileSystemType = FST_EXT3;

      //Terminate at the first space after the device name
      q = strstr(p, " ");
      if(q) *q = '\0';
      strcpy(DeviceName, p);

      //Extract the mount point
      p = strstr(q + 1, "/");
      q = strstr(p, " ");
      if(q) *q = '\0';
      strcpy(OrigMountPoint, p);

      TAP_MemFree(Buffer);
      return TRUE;
    }
  }

  TAP_Print("TMSMount: no /dev/sd? found\n");

  TAP_MemFree(Buffer);
  return FALSE;
}

void GetDestinationMountPoint(void)
{
  int                   fd;
  dword                 fs;
  char                 *INI, *EOL;
  char                  Buffer[255];

  UserMountPoint[0] = '\0';
  TAP_SPrint(Buffer, "%s/%s", OrigMountPoint, INIFILE);

  fd = open(Buffer, O_RDONLY);
  if(fd > 0)
  {
    fs = get_filesize(Buffer);
    INI = TAP_MemAlloc(fs + 1);
    TAP_SPrint(INI, "/mnt/hd");
    read(fd, &INI[strlen(INI)], fs);
    close(fd);

    EOL = strpbrk(INI, "\x0d\x0a\x00");
    if(EOL)
    {
      *EOL = '\0';
      strcpy(UserMountPoint, INI);

      TAP_Print("TMSMount: '%s'\n", UserMountPoint);
    }
    TAP_MemFree(INI);
  }
  else
  {
    TAP_Print("TMSMount: Failed to open %s\n", Buffer);
    strcpy(UserMountPoint, DEFAULTMOUNTPOINT);
  }
}

bool Remount(void)
{
  char                  buffer[512];
  int                   ret;

  TAP_PrintNet("Mounting drive\n");
  ret = mkdir(UserMountPoint, 0777);
  TAP_PrintNet("  mkdir(%s) = %d\n", UserMountPoint, ret);

  switch(FileSystemType)
  {
    case FST_Normal:
    {
      ret = mount(DeviceName, UserMountPoint, "jfs", 0, "");
      TAP_PrintNet("  mount(%s, %s, %s, %x, %s) = %d\n", DeviceName, UserMountPoint, "jfs", 0, "", ret);

      break;
    }

    case FST_NTFS:
    {
      TAP_SPrint(buffer, "/root/ntfs-3g %s %s", DeviceName, UserMountPoint);
      ret = system(buffer);
      TAP_PrintNet("  %s = %d\n", buffer, ret);

      break;
    }

    case FST_VFAT:
    {
      ret = mount(DeviceName, UserMountPoint, "vfat", 0, "");
      TAP_PrintNet("  mount(%s, %s, %s, %x, %s) = %d\n", DeviceName, UserMountPoint, "vfat", 0, "", ret);

      break;
    }

    case FST_EXT2:
    {
      ret = mount(DeviceName, UserMountPoint, "ext2", 0, "");
      TAP_PrintNet("  mount(%s, %s, %s, %x, %s) = %d\n", DeviceName, UserMountPoint, "ext2", 0, "", ret);

      break;
    }

    case FST_EXT3:
    {
      ret = mount(DeviceName, UserMountPoint, "ext3", 0, "");
      TAP_PrintNet("  mount(%s, %s, %s, %x, %s) = %d\n", DeviceName, UserMountPoint, "ext3", 0, "", ret);

      break;
    }
  }

  return (ret == 0);
}

void Unmount(void)
{
  int                   ret;

  TAP_PrintNet("Unmounting drive\n");
  ret = umount(UserMountPoint);
  TAP_PrintNet("  umount(%s) = %d\n", UserMountPoint, ret);
  TAP_Delay(30);

  ret = rmdir(UserMountPoint);
  TAP_PrintNet("  rmdir(%s) = %d\n", UserMountPoint, ret);
}

dword TMSCommander_handler(dword param1)
{
  switch (param1)
  {
    case TMSCMDR_Capabilities:
      return (dword)(TMSCMDR_CanBeStopped);

    case TMSCMDR_Menu:
      return TMSCMDR_NotOK;

    case TMSCMDR_Stop:
      Unmount();
      TAP_Exit();
      return TMSCMDR_OK;

    default:
      return TMSCMDR_UnknownFunction;
  }
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
  static dword          *__nExtPartition;

  (void) event;
  (void) param2;

  if(OSDMenuInfoBoxIsVisible()) OSDMenuEvent(NULL, NULL, NULL);

  switch(State)
  {
    case ST_Init:
    {
      __nExtPartition = (dword*)FIS_vnExtPartition();
      if(!__nExtPartition)
      {
        OSDMenuInfoBoxShow(PROGRAM_NAME " " VERSION, "Variable nExtPartition nicht gefunden", 300);
        State = ST_ShowFailMessage;
      }

      State = ST_WaitForDisk;
      break;
    }

    case ST_WaitForDisk:
    {
      if(TAP_GetTick() > WaitTimeout) State = ST_Exit;
      if(*__nExtPartition > 0) State = ST_MountDisk;

      break;
    }

    case ST_MountDisk:
    {
      if(GetCurrentMountInfo())
      {
        GetDestinationMountPoint();
        if(Remount())
        {
          if(GetUptime() > 6000)
          {
            TAP_SPrint(Buffer, "Erfolgreich auf \n'%s'\n eingehngt", UserMountPoint);
            OSDMenuInfoBoxShow(PROGRAM_NAME " " VERSION, Buffer, 300);
            State = ST_ShowSuccessMessage;
          }
          else
          {
            State = ST_Idle;
          }
        }
        else
        {
          if(GetUptime() > 6000)
          {
            OSDMenuInfoBoxShow(PROGRAM_NAME " " VERSION, "Mount ist fehlgeschlagen", 300);
            State = ST_ShowFailMessage;
          }
          else
          {
            State = ST_Exit;
          }
        }

        break;
      }

      if(GetUptime() > 6000)
      {
        OSDMenuInfoBoxShow(PROGRAM_NAME " " VERSION, "USB/eSATA-Disk nicht angeschlossen?", 300);
        State = ST_ShowFailMessage;
      }
      else
      {
        State = ST_Exit;
      }

      break;
    }

    case ST_ShowSuccessMessage:
    {
      if(!OSDMenuInfoBoxIsVisible()) State = ST_Idle;
      break;
    }

    case ST_ShowFailMessage:
    {
      if(!OSDMenuInfoBoxIsVisible()) State = ST_Exit;
      break;
    }

    case ST_Idle:
    {
      break;
    }

    case ST_Exit:
    {
      TAP_Exit();
      break;
    }
  }

  if(event == EVT_STOP)
  {
    Unmount();
    TAP_Exit();
  }

  return param1;
}

int TAP_Main(void)
{

  WaitTimeout = TAP_GetTick() + WAITTIMEOUT;
  return 1;
}
